/*
 * POSIX basename utility
 * Taylor Gurney, 2024
 * Public domain.
 */

/* for fputs */
#include <stdio.h>

/* for strlen, strrchr, memcmp */
#include <string.h>

/* for exit */
#include <stdlib.h>

#include "util.h"

static char* argv0;

static void usage(void) {
    fputs("usage: ", stderr);
    fputs(argv0, stderr);
    fputs(" <string> [suffix]\n", stderr);
    exit(1);
}

static size_t rmslashs(char* s, size_t len) {
    /* remove all trailing slash */
    while (len && (s[len - 1] == DIRCHR)) {
        len--;
        s[len] = '\0';
    }
    if (!len) {
        puts(DIRSTR);
        exit(0);
    }
    return len;
}

int main(int argc, char* argv[]) {
    size_t len;
    char* last = NULL;
    char* s;
    char* suffix = "";
    size_t suffixlen;
    argv0 = argv[0];
    shift();
    dashdash();
    if (argc > 2) {
        usage();
    }
    if (argc == 2) {
        suffix = argv[1];
    }
    s = argv[0];
    len = strlen(s);
    suffixlen = strlen(suffix);
    /* If empty string, then empty string. */
    if (!len) {
        fputs("\n", stdout);
        return 0;
    }
    len = rmslashs(s, len);
    /* return '/' if no slashes left */
    last = strrchr(s, DIRCHR);
    if (last != NULL) {
        s = last + 1;
    }
    len = strlen(s);
    rmslashs(s, len);
    if (suffix[0] && len > suffixlen) {
        if (memcmp(suffix, &(s[len - suffixlen]), suffixlen) == 0) {
            s[len-suffixlen] = '\0';
        }
    }
    puts(s);
    return 0;
}
