/*
 * POSIX cat utility
 * Taylor Gurney, 2024.
 * Public domain.
 */

/* Implemented using standard C only, so unbuffered I/O is really
 * slow and therefore not used by default. */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "util.h"

static int unbuf = 0;
static char dasharg[] = {'-', '\0'};
static char* dash[] = {dasharg, NULL};
static char buf[1024];

static char* argv0;

static void emsg(char* s) {
    fputs(argv0, stderr);
    fputs(": ", stderr);
    perror(s);
}

static void usage(void) {
    fputs(""
"usage: cat [-u] [file ...]\n"
"  -u    unbuffered output\n",
stderr);
    exit(1);
}

int main(int argc, char *argv[]) {
    FILE* f = NULL;
    size_t siz = 1024;
    size_t got = 0;
    int rval = 0;
    /* parse command line */
    argv0 = argv[0];
    shift();
    for (; argc && argv[0][0] == '-' && argv[0][1]; shift()) {
        if (argv[0][1] == '-' && !argv[0][2]) {
            shift();
            break;
        }
        for (argv[0]++; argv[0][0]; argv[0]++) {
            switch (argv[0][0]) {
            case 'u':
                unbuf = 1;
                siz = 1;
                break;
            default:
                usage();
            }
        }
    }
    /* default to stdin */
    if (!argc) {
        argc = 1;
        argv = dash;
    }
    /* do it */
    for (; *argv; argc--, argv++) {
        if (strcmp(*argv, "-") != 0) {
            f = fopen(*argv, "rb");
            if (f == NULL) {
                emsg(*argv);
                continue;
            }
        } else {
            f = stdin;
        }
        if (unbuf) {
            setbuf(f, NULL);
        }
        do {
            got = fread(buf, 1, siz, f);
            if (got) {
                fwrite(buf, 1, got, stdout);
            }
        } while (got == siz);
        if (ferror(f)) {
            perror(*argv);
        }
        if (strcmp(*argv, "-") != 0) {
            fclose(f);
        } else {
            clearerr(f);
        }
    }
    return rval;
}
