/*
cxd - hex dump colorizer
Taylor Gurney, 2022-2024.
Public domain.

This is a colorizer for hex dumps produced by "xxd" as well as my own
"xd". It's written in C89.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "util.h"

#define GRAY "\033[1;30m"
#define WHITE "\033[1;37m"
#define GREEN "\033[1;32m"
#define CYAN "\033[1;36m"
#define BLUE "\033[1;34m"
#define MAGENTA "\033[1;35m"
#define RED "\033[1;31m"
#define YELLOW "\033[1;33m"
#define BROWN "\033[0;33m"
#define NO_COLOR "\033[0m"

static char* argv0;

static char* get_color(char hi, char lo) {
    if (hi == '0' && lo == '0') return GRAY;
    if (hi == '0' || hi == '1') return WHITE;
    if (hi == '2' || hi == '3') return GREEN;
    if (hi == '4' || hi == '5') return CYAN;
    if (hi == '6' || hi == '7') return BLUE;
    if (hi == '8' || hi == '9') return MAGENTA;
    if (hi == 'a' || hi == 'b' || hi == 'A' || hi == 'B') return RED;
    if (hi == 'c' || hi == 'd' || hi == 'C' || hi == 'D') return YELLOW;
    if (hi == 'e' || hi == 'f' || hi == 'E' || hi == 'F') return BROWN;
    return NO_COLOR;
}

static int xdigit(char c) {
    return ((c >= '0' && c <= '9')
         || (c >= 'a' && c <= 'f')
         || (c >= 'A' && c <= 'F'));
}

#define SIZ 256
int main(int argc, char* argv[]) {
    char buf[SIZ];
    char* ptr;
    char hi;
    char lo;
    char* thiscolor;

    argv0 = argv[0];

    if (argc > 1) {
        fputs("usage: ", stderr);
        fputs(argv0, stderr);
        fputs("\n", stderr);
        exit(1);
    }
    while (fgets(buf, SIZ, stdin)) {
        ptr = buf;
        /* output address part unchanged */
        for (; *ptr; ptr++) {
            if (*ptr == ':') {
                break;
            } else {
                putchar(*ptr);
            }
        }
        /* output hex dump part colorized */
        for (; *ptr; ptr++) {
            hi = *ptr;
            lo = *(ptr + 1);
            if (hi == ' ' && lo == ' ') {
                putchar(hi);
                putchar(lo);
                ptr += 2;
                break;
            }
            if (xdigit(hi) && xdigit(lo)) {
                thiscolor = get_color(hi, lo);
                fputs(thiscolor, stdout);
                fputc(hi, stdout);
                fputc(lo, stdout);
                fputs(NO_COLOR, stdout);
                ptr++;
            } else {
                putchar(hi);
            }
        }
        /* output the rest uncolorized */
        for (; *ptr; ptr++) {
            putchar(*ptr);
        }
    }
    return 0;
}
